%% Written bu CK on 19.02.2020
% It carries out ML-ELM defined in 2018 ML-KELM paper (below) to solve a wind estimation problem
   %  C. M. Wong, C. M. Vong, P. K. Wong and J. Cao, "Kernel-Based Multilayer Extreme Learning Machines for Representation Learning," 
   %  in IEEE Transactions on Neural Networks and Learning Systems, vol. 29, no. 3, pp. 757-762, March 2018.
clear all;
close all;
clc;
warning off
Nnode=15 ; %The number of neurons in each layer  
C=50; % Regularization factor
L=3; % number of layer in the network
DenSay=30; % number of runs

    %% prepare the inputs/output for testing
     load WindData %Loading data;
     N=size(X,2);%the number of samples=N, X comes from the loaded data
     Nt=size(Xt,2);%the number of samples for test data
     time=[];
     RMSEtrain=[]; RMSEtest=[]; % Cost array in terms of RMSE 
     ite=1;
    while ite<=DenSay  %%%LOOP for numerical experiments 
        tstart = tic;   % set clock time
        W=[]; b=[]; Beta=[]; 
        Xegt=X';  %for training set
        Xtemp=Xegt;
        Xtest=Xt'; %for testing set
           %% SLFN structural parameters
            n=size(X,1); %the number of external inputs
            out_size=size(Yd_train,2);%the number of output
            TrFcn = 'logsig';%transfer func of the node
            
            %% Stage:1 Autoencoders learning or representation learning
               for m=1:L-1 %do it for each layer
                  %% construction of mth SLFN and initial assigmants 
                    W=randn(Nnode,size(Xtemp,2))'; %input weights of hidden nodes
                    W=orth( W.' ).';% orthogonal random assigment
                    iw{m}=W;% collect the input weights
                    b=randn(Nnode,1)';
                    b=orth(b.').';  %orthogonal biases of neurons
                    ib{m}=b;  % collect neuron biases 
                     %% construction of H matrice
                     H=feval(TrFcn,Xtemp*W+repmat(b,N,1));  
                     Beta=H'*inv(eye(size(H*H'))/C+H*H')*Xtemp;
                     iBeta{m}=Beta;
                     Xtemp=(Xtemp*Beta');  
                end % for m

     %% Stage:2 Training of the last layer of ELM
              % H computation for the last layer
               Wfinal=randn(Nnode,size(Xtemp,2))'; %input weights of hidden nodes
               Wfinal=orth(Wfinal.').';% orthogonal random assigment
               bfinal=randn(Nnode,1)';  %biases of neurons
               bfinal=orth(bfinal.').';
               Hfinal=feval(TrFcn,Xtemp*Wfinal+repmat(bfinal,N,1)); 
               Beta_final=Hfinal'*inv(eye(size(Hfinal*Hfinal'))/C+Hfinal*Hfinal')*Yd_train;
            time(ite)= toc(tstart);



                      % PERFORMANCE FOR TRAINING DATA with founded output weight (Beta)parameters 
                         Ytrain= Hfinal*Beta_final;
                      %Cost computatiton for training 
                         Cost_train=sqrt(mse(Ytrain,Yd_train)); 
                      %%ploting
                         figure(2);
                         subplot(2,2,1), 
                         plot(Ytrain,'b'),
                         title('Training performance of ML-ELM system');
                         hold on
                         plot(Yd_train,'k'); hold off
                         xlabel('k'), ylabel('y(k)'); legend('obtained', 'desired');  
                          axis([1 length(Ytrain) -0.1 1.1]);

                         subplot(2,2,3); 
                         plot(Ytrain,Yd_train,'b.'), title('Training performance in terms of scatter plot');
                         axis([-0.1 1.1 -0.1 1.1]);

                     % PERFORMANCE FOR TESTING DATA with founded parameters 
                         Xmt=Xtest;
                        for m=1:L-1 %do it for each layer
                             Xmt=Xmt*(iBeta{1,m})';  
                        end % for m

                        HTfinal=feval(TrFcn,(Xmt*Wfinal+repmat(bfinal,Nt,1))); 
                        Ytest= HTfinal*Beta_final;
                 %Cost computatiton for testing 
                        Cost_test=sqrt(mse(Ytest,Yd_test));
                         %ploting
                             figure(2);
                             subplot(2,2,2), 
                             plot(Ytest,'r'),
                             title('Testing performance of ML-ELM system');
                             hold on
                             plot(Yd_test,'k'); hold off
                             xlabel('k'), ylabel('y(k)'); legend('obtained', 'desired');
                             axis([1 length(Yd_test) -0.1 1.1]);

                              subplot(2,2,4); 
                              plot(Ytest,Yd_test,'r.'), title('Testing performance in terms of scatter plot');
                              axis([-0.1 1.1 -0.1 1.1]);
                              %pause(.1);

     RMSEtrain(ite)=Cost_train; RMSEtest(ite)=Cost_test; %Cost saving in an array
     ite=ite+1; %iteration increment
    end % while ite 
    % displaying the training time metrics 
    disp('the elapsed time metrics for training'); 

    average_time=mean(time); 
    best_time=min(time);     
    worst_time=max(time);    
    std_dev_time=std(time);  
    
    disp('[   Mean      Best      Worst     StdDev]'); 
    Mtrc=[average_time best_time worst_time std_dev_time];
    disp(Mtrc);

    disp('The metrics for training data'); 
    average_cost=mean(RMSEtrain); 
    best_cost=min(RMSEtrain);     
    worst_cost=max(RMSEtrain);    
    std_dev_cost=std(RMSEtrain);  
    
    disp('[  MeanRMSE  BestRMSE  WorstRMSE   StdDev]'); 
    Mtrc=[average_cost best_cost worst_cost std_dev_cost];
    disp(Mtrc);

    disp('The metrics for testing data'); 
    average_cost=mean(RMSEtest);  
    best_cost=min(RMSEtest);      
    worst_cost=max(RMSEtest);     
    std_dev_cost=std(RMSEtest);   
    disp('[  MeanRMSE  BestRMSE  WorstRMSE   StdDev]'); 
    Mtrc=[average_cost best_cost worst_cost std_dev_cost];
    disp(Mtrc);
    
   

